#ifndef POLYABSTR_HPP_INCLUDED
#define POLYABSTR_HPP_INCLUDED

#include <map>

template <class Coeff>
struct Polynom {
private:
  std::map<int, Coeff> monoms;
public:
  void zero();
  void set_coeff(int d, Coeff co);
  Coeff coeff(int d) const;
  int degree() const;
  bool is_zero() const;
};


typedef Polynom<float> Polynome;


/* Calcule x^degree
 * @x Valeur de X
 * @degree Puissance
 *
 * @return X^degree
 */
float Puissance(float x, int degree);


/////////////////////////////////////////////////////////////////////////////////
//                       Fonction du type abstrait                             //
/////////////////////////////////////////////////////////////////////////////////


/* Unitialise un nouveau polynome à zero.
 * @p Polynome à initialiser
 */
void  PolynomeNul(Polynome &p);


/* Modifier le coefficient d'un polynome
 * @p Polynome à modifier
 * @d Degree pour lequel le coefficient doit être modifié
 * @co Nouveau coefficient
 */
void  ModifierCoeffPoly(Polynome &p, int d, float co);


/* Récupère le degré du polynome
 * @p Polynome interrogé
 *
 * @return Degré du polynome
 */
int   DegrePoly(const Polynome &p);


/* Récupère un coefficient du polynome
 * @p Polynome interrogé
 * @d Degré du monome dont on veut connaître le coefficient
 *
 * @return Coefficient du monome voulu
 */
float CoeffPoly(const Polynome &p, int d);


/* Teste si un polynome est nul ou pas
 *
 * @return True si le polynome est nul, false sinon
 */
bool  EstNulPoly(const Polynome &p);


/* Évalue la valeur d'un polynome
 * @p Polynome
 * @v Valeur de X
 *
 * @return Valeur du polynome au point considéré
 */
float ValeurPoly(const Polynome &p, float v);


#endif // POLYABSTR_HPP_INCLUDED
