#include <iostream>
#include "PolyAbstr.hpp"

/////////////////////////////////////////////////////////////////////////////////
//                   Utilisation du type abstrait : affichages                 //
/////////////////////////////////////////////////////////////////////////////////

using namespace std;

/* Affiche le monome d'un polynome
 * @i Degréé du monome
 * @co Coefficient du monome
 * @premier Est-ce le premier monome affiché pour un polynome ?
 */
void AfficheMonome(int i, float co, bool premier) {
  if (co != 0) {
    // affiche le signe
    if (co > 0 && not premier) {
      cout << " + ";
    }
    if (co < 0) {
      cout << " - ";
      co = -co;
    }
    // affiche le coefficient
    if ((co != 1) || (i == 0))
      cout << co;
    // affiche la puissance de X
    if (i > 1) cout << "X^" << i;
    else if (i == 1) cout << "X";
  }
}

/* Affiche un polynome dans la console
 * @p Polynome à afficher
 */
void AffichePoly(Polynome p) {
  int d, i;
  if (EstNulPoly(p)) cout << 0 << endl;
  else {
    d = DegrePoly(p);
    AfficheMonome(d, CoeffPoly(p, d), true);
    for (i = d-1; i >= 0; i--)
      AfficheMonome(i, CoeffPoly(p, i), false);
    cout << endl;
  }
}

/////////////////////////////////////////////////////////////////////////////////
//             Utilisation du type abstrait : fonction vue en cours            //
/////////////////////////////////////////////////////////////////////////////////

/* Multiple un polynome par une constante
 * @p Polynome à multiplier
 * @a Constante
 */
void MultPolyConst(Polynome &p, float a) {
  int d, i;
  float c;

  if (a == 0)
    PolynomeNul(p);
  else {
    d = DegrePoly(p);
    for (i=0; i<=d; i++) {
      c = CoeffPoly(p, i);
      ModifierCoeffPoly(p, i, c*a);
    }
  }
}

/* Dérive un polynome
 * @p Polynome à dériver
 * @res Stocke le résultat de la dérivation
 */
void DeriveePoly(Polynome p, Polynome &res) {
  int d, i;
  d = DegrePoly(p);
  PolynomeNul(res);
  for (i=0; i<d; i++) {
    ModifierCoeffPoly(res, i, (i+1)*CoeffPoly(p, i+1));
  }
}

/////////////////////////////////////////////////////////////////////////////////
//               Utilisation du type abstrait : fonction à écrire              //
/////////////////////////////////////////////////////////////////////////////////

float EvalPoly(Polynome p, float x);
float EvalHornerPoly(Polynome p, float x);
void PolynomeCoeffEgaux(Polynome &p, int degree, float coeff);
void ProduitPolyXn(Polynome p, int n, Polynome &res);
void ProduitPoly(Polynome p, Polynome q, Polynome &res);
void PuissancePoly(Polynome p, int n, Polynome &res);

int main() {
  Polynome p, q;

  PolynomeNul(p);
  ModifierCoeffPoly(p, 5,  4.);
  ModifierCoeffPoly(p, 2, -5.);
  ModifierCoeffPoly(p, 1,  1.);
  ModifierCoeffPoly(p, 0, -1.);

  cout << "Le polynome p est : ";
  AffichePoly(p);


  cout << "Le polynome 3*p est : ";
  q = p;
  MultPolyConst(q, 3.);
  AffichePoly(q);

  cout << "La derivee de p est : ";
  DeriveePoly(p, q);
  AffichePoly(q);

  return 0;
}
